//===================================CISCKER====================================
//====================================CCR=======================================
//------------------------------------------------------------------------------
//File Name:	CCR.v
//Purpose:		Decodes CCR bits from opcode, operands and result
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module CCR (
	input [15:0] r,		//alu result
	input [2:0] a,		//operand a
	input [1:0] b,		//operand b
	input [7:0] opcode,
	input clk,				// CCR clock
	input ccr_en,			// CCR enable
	input int,				//Temp
	output [5:0] CCR		// CCR bus line
	);

//////////////////////////////////////////////////////////////////////////////////////////////////
// Register declaratios
//////////////////////////////////////////////////////////////////////////////////////////////////				
reg V; 	//Overflow
reg H;	//Half Carry
reg I;	//Interrupt Mask
reg N;	//Sign (Negative)
reg Z;	//Zero
reg C;	//Carry	

//////////////////////////////////////////////////////////////////////////////////////////////////
// wire declaratios
//////////////////////////////////////////////////////////////////////////////////////////////////
wire a0 	= a[0];
wire a7		= a[1];
wire a15	= a[2];
wire b7		= b[0];
wire b15 	= b[1];

//////////////////////////////////////////////////////////////////////////////////////////////////
// V - overflow
//////////////////////////////////////////////////////////////////////////////////////////////////
always@(posedge clk)
	if (~ccr_en)
		V <= V;
	else 
		case (opcode)
			8'h70,8'h71,8'h72,8'h77,8'h78,
			8'h80,8'h81,8'h82,8'h87,8'h88,
			8'h90,8'h91,8'h92,8'h97,8'h98,
			8'hA0,8'hA1,8'hA2,8'hA7,8'hA8,
			8'hB0,8'hB1,8'hB2,8'hB7,8'hB8,
			8'hC0,8'hC1,8'hC2,8'hC7,8'hC8,
			8'hD0,8'hD1,8'hD2,8'hD7,8'hD8,
			8'h3E,8'h3F,8'h4E:				V <= a7 & ~b7 & ~r[7] | ~a7 & b7 & r[7];	
			
			8'h32,8'h42,8'h52,8'h62:		V <= ~a7&r[7];	//INC
			                            
			8'h02,8'h03,
			8'h33,8'h43,8'h53,8'h63:		V <= ~r[7]&a7;	//DEC
			
			8'h34,8'h44,8'h54,8'h64,
			8'h35,8'h45,8'h55,8'h65,
			8'h36,8'h46,8'h56,8'h66,
			8'h37,8'h47,8'h57,8'h67,
			8'h38,8'h48,8'h58,8'h68:		V <= N ^ C; //ASL
						
			8'h73,8'h83,8'h93,8'hA3,8'hB3,8'hC3,8'hD3,8'hE3,8'hF3,
			8'h74,8'h84,8'h94,8'hA4,8'hB4,8'hC4,8'hD4,8'hE4,8'hF4,
			8'h75,8'h85,8'h95,8'hA5,8'hB5,8'hC5,8'hD5,8'hE5,8'hF5,
			8'h76,8'h86,8'h96,8'hA6,8'hB6,8'hC6,8'hD6,8'hE6,8'hF6,
			8'h39,8'h49,8'h59,8'h69,8'h79,8'h89,8'h99,8'hA9,8'hB9,
			8'h7A,8'h9A,8'hAA,8'hBA,8'hDA,8'hEA,8'hFA,
			8'h3C,8'h4C,8'h5C,8'h6C,8'h5D,8'h6D,
			8'h31,8'h41,8'h51,8'h61,8'h0A:	V <= 1'b0;
			
			8'hCC,8'hDC,8'hEC,8'hFC,
			8'h7E,8'h8E,8'h9E,8'hAE,8'hBE,8'hCE,8'hDE,8'hEE,8'hFE,
			8'h7D,8'h7F,8'h8F,8'h9F,8'hAF:	V <= a15 & ~b15 & ~r[15] | ~a15 & b15 & r[15];
			
			8'h30,8'h40,8'h50,8'h60:		V <= r[7]&a7;
			
			8'h0B: 							V <= 1'b1;
			
			8'h2D:							V <= r[9];//RTI
			
			default:						V <= V;
		endcase
//////////////////////////////////////////////////////////////////////////////////////////////////		
// C - Carry
//////////////////////////////////////////////////////////////////////////////////////////////////
always@(posedge clk)
	if (~ccr_en)
		C <= C;
	else 
		case (opcode)
			8'h70,8'h71,8'h72,8'h77,8'h78,
			8'h80,8'h81,8'h82,8'h87,8'h88,
			8'h90,8'h91,8'h92,8'h97,8'h98,
			8'hA0,8'hA1,8'hA2,8'hA7,8'hA8,
			8'hB0,8'hB1,8'hB2,8'hB7,8'hB8,
			8'hC0,8'hC1,8'hC2,8'hC7,8'hC8,
			8'hD0,8'hD1,8'hD2,8'hD7,8'hD8,
			8'h3E,8'h3F,8'h4E,
			8'h32,8'h42,8'h52,8'h62,
			8'h33,8'h43,8'h53,8'h63:		C <= ~a7&b7 | b7&r[7] | r[7]&~a7;
			
			8'h34,8'h44,8'h54,8'h64,
			8'h35,8'h45,8'h55,8'h65,
			8'h36,8'h46,8'h56,8'h66:		C <= a0;
			
			8'h5A, 8'h6A:					C <= Z;			//BRSET
			8'h5B, 8'h6B:					C <= ~Z;		//BRCLR
			
			8'h37,8'h47,8'h57,8'h67,
			8'h38,8'h48,8'h58,8'h68:		C <= a7;
			
			8'hCC,8'hDC,8'hEC,8'hFC,
			8'h7E,8'h8E,8'h9E,8'hAE,8'hBE,
			8'hCE,8'hDE,8'hEE,8'hFE,
			8'h7D,8'h7F,8'h8F,8'h9F,8'hAF:	C <= ~a15&b15 | b15&r[15] | r[15]&~a15;
			
			8'h30,8'h40,8'h50,8'h60:		C <=~(r==16'b0);
			
			8'h0C:							C <= 1'b0;
			
			8'h0D: 							C <= 1'b1;
			
			8'h2D:							C <= r[8];//RTI
			
			default:						C <= C;
			/*
			8'h00,8'h04,8'h05,8'h06,8'h07,
			8'h10,8'h11,8'h12,8'h13,8'h14,8'h15,8'h16,8'h17,
			8'h18,8'h19,8'h1A,8'h1B,8'h1C,8'h1D,8'h1E,8'h1F,
			8'h26,8'h27,8'h28,8'h29,8'h2A,8'h2B,8'h2C,8'h2E,8'h2F,
			8'hC9,8'hD9,8'hE9,8'hF9,
			8'h73,8'h83,8'h93,8'hA3,8'hB3,8'hC3,8'hD3,8'hE3,8'hF3,
			8'h74,8'h84,8'h94,8'hA4,8'hB4,8'hC4,8'hD4,8'hE4,8'hF4,
			8'h75,8'h85,8'h95,8'hA5,8'hB5,8'hC5,8'hD5,8'hE5,8'hF5,
			8'h76,8'h86,8'h96,8'hA6,8'hB6,8'hC6,8'hD6,8'hE6,8'hF6,
			8'h39,8'h49,8'h59,8'h69,8'h79,8'h89,8'h99,8'hA9,8'hB9,
			8'h7A,8'h9A,8'hAA,8'hBA,8'hDA,8'hEA,8'hFA,
			8'h3C,8'h4C,8'h5C,8'h6C,8'h5D,8'h6D,
			8'h31,8'h41,8'h51,8'h61,*/
		endcase
	
//////////////////////////////////////////////////////////////////////////////////////////////////
// Z - Zero
//////////////////////////////////////////////////////////////////////////////////////////////////
always@(posedge clk)	
	if (~ccr_en)
		Z <= Z;
	else
		case (opcode)
			8'h00,8'h04,8'h05,8'h06,8'h07,
			8'h10,8'h11,8'h12,8'h13,8'h14,8'h15,8'h16,8'h17,
			8'h18,8'h19,8'h1A,8'h1B,8'h1C,8'h1D,8'h1E,8'h1F,
			8'h26,8'h27,8'h28,8'h29,8'h2A,8'h2B,8'h2C,8'h2E,8'h2F,
			8'hC9,8'hD9,8'hE9,8'hF9:		Z <= Z;
			
			8'h2D:							Z <= r[10];//RTI
			
			default:						Z <=(r==16'b0);
		endcase
//////////////////////////////////////////////////////////////////////////////////////////////////
// N - Sign (Negative)
//////////////////////////////////////////////////////////////////////////////////////////////////		
always@(posedge clk)
	if (~ccr_en)
		N <= N;
	else
		case (opcode)
			8'hCC,8'hDC,8'hEC,8'hFC,
			8'h7E,8'h8E,8'h9E,8'hAE,8'hBE,8'hCE,8'hDE,8'hEE,8'hFE,
			8'h7D,8'h7F,8'h8F,8'h9F,8'hAF:	N <= r[15];
			
			8'h00,8'h02,8'h03,8'h04,8'h05,8'h06,8'h07,
			8'h10,8'h11,8'h12,8'h13,8'h14,8'h15,8'h16,8'h17,
			8'h18,8'h19,8'h1A,8'h1B,8'h1C,8'h1D,8'h1E,8'h1F,
			8'h26,8'h27,8'h28,8'h29,8'h2A,8'h2B,8'h2C,8'h2E,8'h2F,
			8'hC9,8'hD9,8'hE9,8'hF9:		N <= N;
			
			8'h2D:							N <= r[11];//RTI
			
			default:						N <= r[7];
		endcase
//////////////////////////////////////////////////////////////////////////////////////////////////
// H - Half Carry
//////////////////////////////////////////////////////////////////////////////////////////////////		
always@(posedge clk)
	if (~ccr_en)
		H <= H;
	else
		case (opcode)
			8'hCC,8'hDC,8'hEC,8'hFC,
			8'h7E,8'h8E,8'h9E,8'hAE,8'hBE,8'hCE,8'hDE,8'hEE,8'hFE,
			8'h7D,8'h7F,8'h8F,8'h9F,8'hAF:	H <= r[7];
			
			8'h00,8'h02,8'h03,8'h04,8'h05,8'h06,8'h07,
			8'h10,8'h11,8'h12,8'h13,8'h14,8'h15,8'h16,8'h17,
			8'h18,8'h19,8'h1A,8'h1B,8'h1C,8'h1D,8'h1E,8'h1F,
			8'h26,8'h27,8'h28,8'h29,8'h2A,8'h2B,8'h2C,8'h2E,8'h2F,
			8'hC9,8'hD9,8'hE9,8'hF9:		H <= H;
			
			8'h2D:							H <= r[13];//RTI
			
			default: 						H <= r[3];
		endcase
			
//////////////////////////////////////////////////////////////////////////////////////////////////
// I - Interrupt
//////////////////////////////////////////////////////////////////////////////////////////////////		
always@(posedge clk)	
	if(~ccr_en)
		I <= I;
	else
		case (opcode)
			8'h0E:							I <= 1'b0; //CLI
			8'h2D:							I <= r[12];//RTI
			8'h0F, 8'h2F:					I <= 1'b1; //SEI,SWI
			default:						I <= I;
		endcase
//////////////////////////////////////////////////////////////////////////////////////////////////
// CCR - Condition Code Register -Bus line
//////////////////////////////////////////////////////////////////////////////////////////////////
assign CCR={H,I,N,Z,V,C};//H,I,N,Z,V,C

endmodule
