//===================================CISCKER====================================
//================================Branch Decoder================================
//------------------------------------------------------------------------------
//File Name:	branch_decoder.v
//Purpose:		Decodes branch signal from condition and opcode
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module branch_decoder(
	//inputs
	input clk_0,
	input final,
	input [7:0]opcode,
	input [5:0] CCR,	//H,I,N,Z,V,C
	//output
	output reg branch
	);
	
wire C = CCR[0]; //Carry
wire V = CCR[1]; //Overflow
wire Z = CCR[2]; //Zero
wire N = CCR[3]; //Negative
wire I = CCR[4]; //Interruption
wire H = CCR[5]; //Half-Carry

//								DABNZ				DBBNZ			BRSET IX			BRCLR IX		BRSET EXT			BRCLR EXT
wire update_branch = final | (opcode==8'h02) | (opcode==8'h03) | (opcode==8'h5A) | (opcode==8'h5B) | (opcode==8'h02) | (opcode==8'h02) ;

always @ (posedge clk_0)
if(update_branch)
	case (opcode)
		
		8'h02:	branch <=	(Z==0);			//DABNZ
		8'h03:	branch <=	(Z==0);			//DBBNZ
		8'h04:	branch <=	1'b0;			//BRN
		8'h05:	branch <=	1'b1;			//BRA
		8'h06:	branch <=	(I==0);			//BIL
		8'h07:	branch <=	(I==1);			//BIH
		
		8'h10:	branch <= 	(H==0);			//BHCC
		8'h11:	branch <= 	(H==1);			//BHCC
		8'h12:	branch <= 	((C|Z)==0);		//BHI
		8'h13:	branch <= 	((C|Z)==1);		//BLS
		8'h14:	branch <= 	(C==0);			//BCC
		8'h15:	branch <= 	(C==1);			//BCS
		8'h16:	branch <= 	(Z==0);			//BNE
		8'h17:	branch <= 	(Z==1);			//BEQ
		8'h18:	branch <= 	(V==0);			//BVC
		8'h19:	branch <= 	(V==1);			//BVS
		8'h1A:	branch <= 	(N==0);			//BPL
		8'h1B:	branch <= 	(N==1);			//BMI
		8'h1C:	branch <= 	((N^V)==0);		//BGE
		8'h1D:	branch <= 	((N^V)==1);		//BLT
		8'h1E:	branch <= 	(Z|(N^V)==1);	//BGT
		8'h1F:	branch <= 	(Z|(N^V)==1);	//BLE
		
		8'h5A:	branch <=	(Z==0);			//BRSET
		8'h5B:	branch <=	(Z==0);			//BRCLR
		8'h6A:	branch <=	(Z==0);			//BRSET
		8'h6B:	branch <=	(Z==0);			//BRCLR
		
		default:branch <=	(1'b0);			//Default

	endcase
	
endmodule
