//===================================CISCKER====================================
//====================================Timer=====================================
//------------------------------------------------------------------------------
//File Name:	timer.v
//Purpose:		Configurable Timer module and overflow interruption signal
//Parameters:	None
//------------------------------------------------------------------------------
//=====================Alfredo Gualdron - Jose P. Pinilla=======================
//=================================Semillero ADT================================
//================================UPB Bucaramanga===============================
//Website:	http://semilleroadt.upbbga.edu.co/xiscker
//E-mail:	semilleroadt@gmail.com
//------------------------------------------------------------------------------
module timer(	input clk, 
					input [1:0] prescaler, 
					input[15:0] period, 
					input enable,
					input reset_tmr,
					output reg int_timer,
					output reg [15:0] counter=0
);


reg [1:0] counter_clk=0;		//Prescaler Counter
reg clk_timer_f=0;				//Prescaler Divided Clock


//Prescaler Selection
reg[7:0] pr;

always@(*)
	case(prescaler)
		2'b00:	pr=8'd00;
		2'b01:	pr=8'd07;
		2'b10:	pr=8'd63;
		2'b11:	pr=8'd255;
		default: pr=8'd00;
	endcase

//Prescaler
always @ (negedge clk)
		if(counter_clk == pr)
			counter_clk <= 0;
		else
			counter_clk <= counter_clk + 1;
	
always@(posedge clk)
	if(counter_clk == pr)
		clk_timer_f <= ~clk_timer_f;
	else
		clk_timer_f<=clk_timer_f;
	
//Timer Counter
always @ (posedge clk_timer_f)
	begin
		if(int_timer|reset_tmr)
			counter <= 0;
		else
			if(enable)
				counter <= counter + 1;
			else
				counter <= counter;
	end

always@(posedge clk)
int_timer <= (counter == period);

endmodule
